% Test routine for Srinivasan's FFT based FOE-from-flow method.
% foe = true focus of expansion in pixel coords
% w = true angular velocity in radians
% DT = time interval for flow, e.g. 1e-3

function [foerr,est] = tst_srinivasan(foe,w,DT,f,escale)
   % image window
   mi = 32; ni = 32;

   % ewin gives window for FOE calculation. Here it surrounds and is
   % escale times larger than the image.
   if nargin<5, escale = 3; end;
   me = floor(escale*mi);
   ne = floor(escale*ni);
   offx = floor((mi-me)/2);
   offy = floor((ni-ne)/2);
   ewin = [offx+1, offx+me; offy+1, offy+ne];

   % Camera calibration
   % f = 1000; % long lens
   % f = 32; % very wide angle
   if nargin<4, f = 32; end;
   K = [f,0,(mi)/2; 0,f,(ni)/2; 0,0,1];
   % K = [1,0,0; 0,1,0; 0,0,1];
   IK = inv(K);

   % Motion parameters and projection matrices
   if nargin<1, foe = K(:,3); end;
   if nargin<2, w = [0;0;0]; end;
   if nargin<3, DT = 1e-3; end;
   dt = -DT*unit_vector(IK*foe);
   dw = DT*w;
   R = quat_to_rot([dw/2; 1]);
   P0 = K*eye(3,4);
   P1 = K*[R,-R*dt];

   % Default dense depth field, random displacements from a
   % fronto-parallel plane
   depth = 1;
   thickness = 1e-2;
   x0 = [1:mi]'*ones(1,ni);
   y0 = ones(mi,1)*[1:ni];
   depths = depth*(ones(mi,ni) + thickness*randn(mi,ni));

   if 0
      % Generate data by explicit projection.  Build a dense 3D field
      % [X0;Y0;Z0] at the given depths from the dense image point field
      % [x0;y0;1] in image 0, then project it into image 1.

      X0 = depths.*(IK(1,1)*x0 + IK(1,2)*y0 + IK(1,3));
      Y0 = depths.*(IK(2,1)*x0 + IK(2,2)*y0 + IK(2,3));
      Z0 = depths.*(IK(3,1)*x0 + IK(3,2)*y0 + IK(3,3));

      x1 = P1(1,1)*X0 + P1(1,2)*Y0 + P1(1,3)*Z0 + P1(1,4);
      y1 = P1(2,1)*X0 + P1(2,2)*Y0 + P1(2,3)*Z0 + P1(2,4);
      z1 = P1(3,1)*X0 + P1(3,2)*Y0 + P1(3,3)*Z0 + P1(3,4);
      x1 = x1./z1;
      y1 = y1./z1;

      % flow calculation
      fx = x1 - x0;
      fy = y1 - y0;
   else
      % Generate flow fields by using 1st order flow model as if it were
      % exact. In the noiseless case this should give a minimal residual
      % error of exactly zero.

      Bx = IK(1,1)*x0 + IK(1,2)*y0 + IK(1,3);
      By = IK(2,1)*x0 + IK(2,2)*y0 + IK(2,3);

      cfx = -(Bx.*By)*dw(1) + (1+Bx.^2)*dw(2) - By*dw(3) + (dt(1) - Bx*dt(3))./depths;
      cfy = -(1+By.^2)*dw(1) + (Bx.*By)*dw(2) + Bx*dw(3) + (dt(2) - By*dt(3))./depths;

      fx = K(1,1)*cfx + K(1,2)*cfy;
      fy = K(2,1)*cfx + K(2,2)*cfy;
   end
   fweight = 1;
   fcov = [];

   % fx,fy
   % mesh(fx); pause; mesh(fy); pause;

   % Call Srinivasan method

   [foerr,est] = srinivasan(ewin,fx,fy,K,fweight,fcov);
   est(4:6) = est(4:6)/DT

   % Mesh or contour plots. These show clear bimodality for
   % data within a few percent of coplanarity.

   xrange = ewin(1,1):ewin(1,2);
   yrange = ewin(2,1):ewin(2,2);
   % e = foerr;
   e = max(-10,log10(abs(foerr)));
   if 1
      z0=min(min(e)); z1=max(max(e));
      % old octave
      % contour(e,z0+[-1:21]*(z1-z0)/20,xrange,yrange);
      % matlab
      contour(xrange,yrange,e,20);
   else
      mesh(xrange,yrange,e); 
   end

%end;

% 32x32 escale=3
% K = [32,0,(mi)/2; 0,32,(ni)/2; 0,0,1];
% [foeerr,est]=tst_srinivas([16;4.001;1],[0;0;0],1e-3);
% [foeerr,est]=tst_srinivas([16;4.000;1],[0;0;0],1e-3);

% [foeerr,est]=tst_srinivas([23.999;16;1],[0;0;0],1e-3);
% [foeerr,est]=tst_srinivas([24.000;16;1],[1;2;3],1e-3);
